@@ -26,7 +26,6 @@
  * demonstrates how to handle NV-CONTROL events.
  */
 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -624,7 +623,6 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_THERMAL_SENSOR_READING),
     MAKE_ENTRY(NV_CTRL_THERMAL_SENSOR_PROVIDER),
     MAKE_ENTRY(NV_CTRL_THERMAL_SENSOR_TARGET),
-    MAKE_ENTRY(NV_CTRL_GPU_CURRENT_PROCESSOR_CLOCK_FREQS),
     MAKE_ENTRY(NV_CTRL_GPU_PCIE_MAX_LINK_SPEED),
     MAKE_ENTRY(NV_CTRL_3D_VISION_PRO_RESET_TRANSCEIVER_TO_FACTORY_SETTINGS),
     MAKE_ENTRY(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL),
@@ -655,8 +653,6 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_FRAMELOCK_DISPLAY_CONFIG),
     MAKE_ENTRY(NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY),
     MAKE_ENTRY(NV_CTRL_USED_DEDICATED_GPU_MEMORY),
-    MAKE_ENTRY(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE),
-    MAKE_ENTRY(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT),
     MAKE_ENTRY(NV_CTRL_DPY_HDMI_3D),
     MAKE_ENTRY(NV_CTRL_BASE_MOSAIC),
     MAKE_ENTRY(NV_CTRL_MULTIGPU_PRIMARY_POSSIBLE),
